'==============================================================================
' Copyright  Micromega Corporation 2002-2007. All rights reserved.
'
' @file:	demo2.bas
' @target:	ARMmite/ARMexpress
'
' This program provides an example of using the uM-FPU V3 floating point
' coprocessor with the ARMmite/ARMexpress using an SPI interface. It takes
' temperature readings from a DS1620 digital thermometer, converts them to
' floating point, displays the temperature in degrees Celsius and degrees
' Fahrenheit, and calculates the minimum and maximum temperatures.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	August 2, 2007
'	- updated for uM-FPU V3.1
'	November 9, 2006
'	- original version
'==============================================================================

'-------------------- uM-FPU V3 definitions -----------------------------------

#include "FPUspi.bas"

'-------------------- DS1620 pin definitions ----------------------------------

const	DS_RST		= 12				' DS1620 reset/enable
const	DS_CLK      = 11				' DS1620 clock
const	DS_DATA     = 10				' DS1620 data

'-------------------- uM-FPU register definitions -----------------------------

const	DegC		= 8					' temperature in degrees Celsius
const	DegC_Min	= 9					' minimum temperature
const	DegC_Max	= 10				' maximum temperature
const	DegF		= 11				' temperature in degrees Fahrenheit
const	F1_8		= 12				' constant 1.8

'==============================================================================
'-------------------- main routine --------------------------------------------
'==============================================================================

Main:
	print
	print "Demo 2"
	print "------"

	gosub Fpu_Reset						' reset the uM-FPU V3 chip
	if status <> SYNC_CHAR then			' check for synchronization
		print "uM-FPU not detected"
		end
	else
		gosub Print_Version				' display the uM-FPU version number
		print
	endif

	gosub Init_DS1620					' initialize DS1620
									
										' load floating point constant 1.8
	SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, F1_8, ATOF, "1.8", 0, FSET0]

	print "              Min   Current   Max"
	print "             -----  -------  -----"


	gosub Read_Temp						' get initial temperature reading
										' initialize MIN/MAX values
	SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, DegC_Min, FSET, DegC]
	SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, DegC_Max, FSET, DegC]

'-------------------- main loop -----------------------------------------------

	do
		gosub Read_Temp					' get temperature reading
										' calculate min/max values
		SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, DegC_Min, FMIN, DegC]
		SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, DegC_Max, FMAX, DegC]

		print "Celsius   ";				' display temperatures in Celsius
		reg = DegC_Min
		gosub Print_TempC
		reg = DegC
		gosub Print_TempC
		reg = DegC_Max
		gosub Print_TempC
		print

		print "Fahrenheit";				' display temperatures in Fahrenheit
		reg = DegC_Min
		gosub Print_TempF
		reg = DegC
		gosub Print_TempF
		reg = DegC_Max
		gosub Print_TempF
		print

		WAIT(2000)						' delay 2 seconds
		print
	loop

'-------------------- Read_Temp -----------------------------------------------

Read_Temp:
	gosub Read_DS1620					' read temperature value
										' send rawTemp to uM-FPU
										' convert to floating point
										' store in register
										' divide by 2 to get degrees Celsius
	SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, DegC, LOADWORD, rawTemp>>8, rawTemp, FSET0, FDIVI, 2]
	return

'-------------------- Print_TempC ---------------------------------------------
' input:  reg         selects the uM-FPU register to display
'------------------------------------------------------------------------------

Print_TempC:
										' select reg as register A
	SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, reg]

	format = 81							' print reg as formatted floating point
	gosub Print_FloatFormat
	return

'-------------------- Print_TempF ---------------------------------------------
' input:  reg         selects the uM-FPU register to convert and display
'------------------------------------------------------------------------------

Print_TempF:    						' degF = reg * 1.8 + 32
	SPIOUT FpuCS, FpuOut, FpuClk, [SELECTA, DegF, FSET, reg, FMUL, F1_8, FADDI, 32]

	format = 81							' print reg as formatted floating point
	gosub Print_FloatFormat
	return

'-------------------- Init_DS1620 ---------------------------------------------

Init_DS1620:
	IO(DS_RST) = 0						' initialize pin states
	IO(DS_CLK) = 1
	WAIT(100)

	OUT(DS_RST) = 1						' configure for CPU control
	SHIFTOUT DS_DATA, DS_CLK, 0, [$0C, $02]
	OUT(DS_RST) = 0
	WAIT(100)

	OUT(DS_RST) = 1						' start temperature conversions
	SHIFTOUT DS_DATA, DS_CLK, 0, [$EE]
	OUT(DS_RST) = 0
	WAIT(1000)							' wait for first conversion
	return

'-------------------- Read_DS1620 ---------------------------------------------

Read_DS1620:
	OUT(DS_RST) = 1						' read temperature value
	SHIFTOUT DS_DATA, DS_CLK, 0, [$AA]
	SHIFTIN  DS_DATA, DS_CLK, 1, [rawTemp\9]
	OUT(DS_RST) = 0						' extend the sign bit
	if (rawTemp AND 0x100) then rawTemp = rawTemp OR 0xFFFFFF00
	return
